from mitogen.core import BytesIO
from mitogen.core import BytesType
from mitogen.core import Pickler
from mitogen.core import Unpickler
from mitogen.core import UnicodeType
from mitogen.core import UnpicklingError
from mitogen.core import b
from mitogen.core import find_deny

import testlib

def dumps(obj, protocol):
    f = BytesIO()
    Pickler(f, protocol).dump(obj)
    return f.getvalue()


def loads(s, find_class):
    return Unpickler(BytesIO(s), find_class).load()


def find_bytes(module, func):
    lookup = {
        ('__builtin__', 'bytes'): BytesType,
        ('_codecs', 'encode'): UnicodeType.encode,
    }
    try:
        return lookup[(module, func)]
    except KeyError:
        raise UnpicklingError


def find_complex(module, func):
    if (module, func) == ('__builtin__', 'complex'): return complex
    raise UnpicklingError


class PicklerTest(testlib.TestCase):
    def test_bytes_toplevel(self):
        pickled = dumps(b('abc'), protocol=2)
        self.assertFalse(b('_codecs') in pickled)
        self.assertFalse(b('encode') in pickled)
        self.assertFalse(b('latin1') in pickled)


class RoundTripTest(testlib.TestCase):
    def assertRoundTrip(self, obj, find_class):
        self.assertEqual(obj, loads(dumps(obj, protocol=2), find_class))

    def test_bytes(self):
        # Top level bytes should not invoke _codecs.decode
        self.assertRoundTrip(b(''), find_deny)
        self.assertRoundTrip(b('abc'), find_deny)

        # Nested bytes still (unavoidably) invokes _codecs.decode
        self.assertRoundTrip([b('')], find_bytes)
        self.assertRoundTrip([b('abc')], find_bytes)


class UnpicklerTest(testlib.TestCase):
    # These types have no dedicated pickle opcodes at this protocol, they use
    # GLOBAL which invokes find_global() or find_class() during unpickling.
    pickled_complex = dumps(1j, protocol=2)
    pickled_frozenset = dumps(frozenset([1]), protocol=2)

    def test_default_denies(self):
        unpickler = Unpickler(BytesIO(self.pickled_complex))
        self.assertRaises(UnpicklingError, unpickler.load)

    def test_explicit_callback_allows(self):
        unpickler = Unpickler(BytesIO(self.pickled_complex), find_complex)
        self.assertEqual(1j, unpickler.load())

    def test_explicit_callback_denies(self):
        unpickler = Unpickler(BytesIO(self.pickled_frozenset), find_complex)
        self.assertRaises(UnpicklingError, unpickler.load)



import sys


if True:
    import in_if_always_true
    from in_if_always_true import x as y, z
else:
    import in_else_never_true
    from in_else_never_true import x as y, z

if sys.version >= (3, 0):
    import in_if_py3
    from in_if_py3 import x as y, z
else:
    import in_else_py2
    from in_else_py2 import x as y, z



# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(
            warning=dict(type=str, default=u''),
        ),
    )
    if module.params['warning']:
        module.warn(module.params['warning'])
    module.exit_json()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# I am an Ansible new-style Python module. I should receive an encoding string.
# See also custom_python_new_style_module, we should be updated in tandem.

import io
import json
import select
import signal
import sys
import warnings

# Ansible 2.7 changed how new style modules are invoked. It seems that module
# parameters are *sometimes* read before the module runs. Modules that try
# to read directly from stdin, such as this, are unable to. However it doesn't
# always fail, influences seem to include Ansible & Python version. As noted
# in ansible.module_utils.basic._load_params() we should probably use that.
# I think (medium confidence) I narrowed the inflection (with git bisect) to
# https://github.com/ansible/ansible/commit/52449cc01a71778ef94ea0237eed0284f5d75582

# This is the magic marker Ansible looks for:
# from ansible.module_utils.

# These timeouts should prevent hard-to-attribute, 2+ hour CI job timeouts.
# Previously this module has waited on stdin forever (timeoutInMinutes=120).
SELECT_TIMEOUT = 5.0    # seconds
SIGNAL_TIMEOUT = 10     # seconds


def fail_json(msg, **kwargs):
    kwargs.update(failed=True, msg=msg)
    print(json.dumps(kwargs, sys.stdout, indent=2, sort_keys=True))
    sys.exit(1)


def sigalrm_handler(signum, frame):
    fail_json("Still executing after SIGNAL_TIMEOUT=%ds" % (SIGNAL_TIMEOUT,))


def usage():
    sys.stderr.write('Usage: %s <input.json>\n' % (sys.argv[0],))
    sys.exit(1)


# Wait SIGNAL_TIMEOUT seconds, exit with failure if still running.
signal.signal(signal.SIGALRM, sigalrm_handler)
signal.alarm(SIGNAL_TIMEOUT)

# Wait SELECT_TIMEOUT seconds, exit with failure if no data appears on stdin.
# TODO Combine select() & read() in a loop, to handle slow trickle of data.
#      Consider buffering, line buffering, `f.read()` vs `f.read1()`.
# TODO Document that sys.stdin may be a StringIO under Ansible + Mitogen.
try:
    inputs_ready, _, _ = select.select([sys.stdin], [], [], SELECT_TIMEOUT)
except (AttributeError, TypeError, io.UnsupportedOperation) as exc:
    # sys.stdin.fileno() doesn't exist or can't return a real file descriptor.
    warnings.warn("Could not wait on sys.stdin=%r: %r" % (sys.stdin, exc))
else:
    if not inputs_ready:
        fail_json("Gave up waiting on sys.stdin after SELECT_TIMEOUT=%ds"
                  % (SELECT_TIMEOUT,))

# Read all data on stdin. May block forever, if EOF is not reached.
input_json = sys.stdin.read()

print("{")
print("  \"changed\": false,")
# v2.5.1. apt.py started depending on this.
# https://github.com/dw/mitogen/issues/210
print("  \"__file__\": \"%s\"," % (__file__,))
# Python sets this during a regular import.
print("  \"__package__\": \"%s\"," % (__package__,))
print("  \"msg\": \"Here is my input\",")
print("  \"input\": [%s]" % (input_json,))
print("}")

# Ansible since 2.7.0/52449cc01a7 broke __file__ and *requires* the module
# process to exit itself. So needless.
sys.exit(0)



import os
import re

import testlib

import mitogen.core
import mitogen.sudo

class PasswordPromptTest(testlib.TestCase):
    def test_matches(self):
        # macOS 26.4.1, en_GB
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'Password:'))
        # Ubuntu 24.04, sudo-ws, en_GB
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo] password for alex: '))
        # Ubuntu 26.04, sudo-rs, en_GB
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo: authenticate] Password: '))

    def test_translated_matches(self):
        # Using French translation(s) as a stand-in for all translations.
        # Debian 9, sudo-ws, LC_ALL=fr_FR.UTF-8 LANG=fr_FR.UTF-8 LANGUAGE=fr_FR.UTF-8
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(u'[sudo] Mot de passe de alex\N{NO-BREAK SPACE}:\N{NO-BREAK SPACE}'.encode('utf-8')))
        # RHEL 8, sudo-ws
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'Mot de passe de US3RN4ME:'))
        # Ubuntu 24.04, sudo-ws, fr_FR.UTF-8
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo] Mot de passe de alex : '))
        # Ubuntu 26.04, sudo-rs, fr_FR.UTF-8
        self.assertTrue(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo: authenticate] Mot de passe : '))


class PasswordPromptInProgressTest(testlib.TestCase):
    def test_empty_password_no_match(self):
        "A zero length password should not match the prompt again."
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'Password:\n'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo: authenticate] Password: \n'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(u'[sudo] Mot de passe de alex\N{NO-BREAK SPACE}:\N{NO-BREAK SPACE}\n'.encode('utf-8')))

    def test_pwfeedback_no_match(self):
        """
        Enabling pwfeedback (default in Ubuntu 26.04) shouldn't cause repeated
        matches of the prompt when '*' is echoed.
        """
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'Password:*'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'Password:*\n'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo: authenticate] Password: *'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(b'[sudo: authenticate] Password: *\n'))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(u'[sudo] Mot de passe de alex\N{NO-BREAK SPACE}:\N{NO-BREAK SPACE}*'.encode('utf-8')))
        self.assertIsNone(mitogen.sudo.PASSWORD_PROMPT_RE.search(u'[sudo] Mot de passe de alex\N{NO-BREAK SPACE}:\N{NO-BREAK SPACE}*\n'.encode('utf-8')))


class ConstructorTest(testlib.RouterMixin, testlib.TestCase):
    sudo_path = testlib.data_path('stubs/stub-sudo.py')

    def run_sudo(self, **kwargs):
        context = self.router.sudo(
            sudo_path=self.sudo_path,
            **kwargs
        )
        argv = eval(context.call(os.getenv, 'ORIGINAL_ARGV'))
        return context, argv


    def test_basic(self):
        context, argv = self.run_sudo()
        self.assertEqual(argv[:4], [
            self.sudo_path,
            '-u', 'root',
            '--'
        ])

    def test_selinux_type_role(self):
        context, argv = self.run_sudo(
            selinux_type='setype',
            selinux_role='serole',
        )
        self.assertEqual(argv[:8], [
            self.sudo_path,
            '-u', 'root',
            '-r', 'serole',
            '-t', 'setype',
            '--'
        ])

    def test_reparse_args(self):
        context, argv = self.run_sudo(
            sudo_args=['--type', 'setype', '--role', 'serole', '--user', 'user']
        )
        self.assertEqual(argv[:8], [
            self.sudo_path,
            '-u', 'user',
            '-r', 'serole',
            '-t', 'setype',
            '--'
        ])

    def test_tty_preserved(self):
        # issue #481
        os.environ['PREHISTORIC_SUDO'] = '1'
        try:
            context, argv = self.run_sudo()
            self.assertEqual('1', context.call(os.getenv, 'PREHISTORIC_SUDO'))
        finally:
            del os.environ['PREHISTORIC_SUDO']


class SudoMixin(testlib.DockerMixin):
    def test_password_required(self):
        ssh = self.docker_ssh(
            username='mitogen__has_sudo',
            password='has_sudo_password',
        )
        ssh.call(os.putenv, 'LANGUAGE', 'fr')
        ssh.call(os.putenv, 'LC_ALL', 'fr_FR.UTF-8')
        e = self.assertRaises(mitogen.core.StreamError,
            lambda: self.router.sudo(via=ssh)
        )
        self.assertIn(mitogen.sudo.password_required_msg, str(e))

    def test_password_incorrect(self):
        ssh = self.docker_ssh(
            username='mitogen__has_sudo',
            password='has_sudo_password',
        )
        ssh.call(os.putenv, 'LANGUAGE', 'fr')
        ssh.call(os.putenv, 'LC_ALL', 'fr_FR.UTF-8')
        e = self.assertRaises(mitogen.core.StreamError,
            lambda: self.router.sudo(via=ssh, password='x')
        )
        self.assertIn(mitogen.sudo.password_incorrect_msg, str(e))

    def test_password_okay(self):
        ssh = self.docker_ssh(
            username='mitogen__has_sudo',
            password='has_sudo_password',
        )
        ssh.call(os.putenv, 'LANGUAGE', 'fr')
        ssh.call(os.putenv, 'LC_ALL', 'fr_FR.UTF-8')
        e = self.assertRaises(mitogen.core.StreamError,
            lambda: self.router.sudo(via=ssh, password='rootpassword')
        )
        self.assertIn(mitogen.sudo.password_incorrect_msg, str(e))


for distro_spec in testlib.DISTRO_SPECS.split():
    # Only debian<version>-test images have translations installed/configured
    if not re.match('debian', distro_spec, re.IGNORECASE):
        continue

    dockerized_ssh = testlib.DockerizedSshDaemon(distro_spec)
    klass_name = 'SudoTest%s' % (dockerized_ssh.distro.capitalize(),)
    klass = type(
        klass_name,
        (SudoMixin, testlib.TestCase),
        {'dockerized_ssh': dockerized_ssh},
    )
    globals()[klass_name] = klass



#!/usr/bin/env python
# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause
from __future__ import absolute_import, division, print_function

import os

import ci_lib
from ci_lib import subprocess

with ci_lib.Fold('machine_prep'):
    os.chdir(ci_lib.IMAGE_PREP_DIR)
    subprocess.check_call('ansible-playbook -c local -i localhost, _user_accounts.yml'.split())
    subprocess.check_call('ansible-playbook -c local -i localhost, macos_localhost.yml'.split())



#!/usr/bin/env python
"""
Print the size of a typical SSH command line and the bootstrap code sent to new
contexts.
"""

import inspect
import sys
import zlib

import mitogen.core
import mitogen.fakessh
import mitogen.fork
import mitogen.master
import mitogen.minify
import mitogen.parent
import mitogen.select
import mitogen.service
import mitogen.ssh
import mitogen.sudo


class Table(object):
    HEADERS = (' ', 'Original', 'Minimized', 'Compressed')
    HEAD_FMT = '{:20} {:^15}  {:^19}  {:^19}'
    ROW_FMT =  '%-20s %6i %5.1fKiB  %5i %4.1fKiB %4.1f%%  %5i %4.1fKiB %4.1f%%'

    def header(self):
        return self.HEAD_FMT.format(*self.HEADERS)


router = mitogen.master.Router()
context = mitogen.parent.Context(router, 0)
options = mitogen.ssh.Options(
    hostname='foo',
    max_message_size=0,
    remote_name='alice@host:1234',
)
conn = mitogen.ssh.Connection(options, router)
conn.context = context

print('SSH command size: %s' % (len(' '.join(conn.get_boot_command())),))
print('Preamble (mitogen.core + econtext) size: %s (%.2fKiB)' % (
    len(conn.get_preamble()),
    len(conn.get_preamble()) / 1024.0,
))
print('')

if '--dump' in sys.argv:
    print(zlib.decompress(conn.get_preamble()))
    exit()


table = Table()
print(table.header())
for mod in (
        mitogen.core,
        mitogen.parent,
        mitogen.fork,
        mitogen.ssh,
        mitogen.sudo,
        mitogen.select,
        mitogen.service,
        mitogen.fakessh,
        mitogen.master,
    ):
    original = inspect.getsource(mod)
    original_size = len(original)
    minimized = mitogen.minify.minimize_source(original)
    minimized_size = len(minimized)
    compressed = zlib.compress(minimized.encode(), 9)
    compressed_size = len(compressed)
    print(
        table.ROW_FMT
    % (
        mod.__name__,
        original_size,
        original_size / 1024.0,
        minimized_size,
        minimized_size / 1024.0,
        100 * minimized_size / float(original_size),
        compressed_size,
        compressed_size / 1024.0,
        100 * compressed_size / float(original_size),
    ))



#!/usr/bin/env python
# Run tests/ansible/all.yml under Ansible and Ansible-Mitogen

from __future__ import print_function

import os
import sys

import ci_lib

with ci_lib.Fold('ansible'):
    os.chdir(ci_lib.ANSIBLE_TESTS_DIR)
    playbook = os.environ.get('PLAYBOOK', 'all.yml')
    ci_lib.run('./run_ansible_playbook.py %s %s',
        playbook, ' '.join(sys.argv[1:]))



import os
import tempfile

import mitogen.core

import testlib

class BlockingIOTest(testlib.TestCase):
    def setUp(self):
        super(BlockingIOTest, self).setUp()
        self.fp = tempfile.TemporaryFile()
        self.fd = self.fp.fileno()

    def tearDown(self):
        self.fp.close()
        super(BlockingIOTest, self).tearDown()

    def test_get_blocking(self):
        if hasattr(os, 'get_blocking'):
            self.assertEqual(
                os.get_blocking(self.fd), mitogen.core.get_blocking(self.fd),
            )
        self.assertTrue(mitogen.core.get_blocking(self.fd) is True)

    def test_set_blocking(self):
        mitogen.core.set_blocking(self.fd, False)
        if hasattr(os, 'get_blocking'):
            self.assertEqual(
                os.get_blocking(self.fd), mitogen.core.get_blocking(self.fd),
            )
        self.assertTrue(mitogen.core.get_blocking(self.fd) is False)




#!/usr/bin/env python
# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause
from __future__ import absolute_import, division, print_function

import sys

import ci_lib
from ci_lib import subprocess

interesting = ci_lib.get_interesting_procs()

with ci_lib.Fold('unit_tests'):
    subprocess.run(
        [
            sys.executable, '-m', 'unittest', 'discover',
            '--start-directory', sys.argv[1],
            '--pattern', '*_test.py',
            '--verbose',
        ],
        check=True,
    )

ci_lib.check_stray_processes(interesting)



import errno
import operator
import os

import mitogen.core
import mitogen.parent
from mitogen.core import b

import testlib


def create_child_using_pipes(args, blocking, preexec_fn=None):
    """
    Create a child process whose stdin/stdout/stderr is connected to a pipe.

    :param list args:
        Program argument vector.
    :param bool blocking:
        If :data:`True`, the sockets use blocking IO, otherwise non-blocking.
    :param function preexec_fn:
        If not :data:`None`, a function to run within the post-fork child
        before executing the target program.
    :returns:
        :class:`PopenProcess` instance.
    """

    parent_rfp, child_wfp = mitogen.core.pipe(blocking)
    child_rfp, parent_wfp = mitogen.core.pipe(blocking)
    stderr_r, stderr = mitogen.core.pipe(blocking=blocking)
    mitogen.core.set_cloexec(stderr_r.fileno())
    try:
        proc = testlib.subprocess.Popen(
            args=args,
            stdin=child_rfp,
            stdout=child_wfp,
            stderr=stderr,
            close_fds=True,
            preexec_fn=preexec_fn,
        )
    except Exception:
        parent_rfp.close()
        parent_wfp.close()
        stderr_r.close()
        raise
    finally:
        child_rfp.close()
        child_wfp.close()
        stderr.close()

    return mitogen.parent.PopenProcess(
        proc=proc,
        stdin=parent_wfp,
        stdout=parent_rfp,
        stderr=stderr_r,
    )


def create_child_using_sockets(args, blocking, size=None, preexec_fn=None):
    """
    Create a child process whose stdin/stdout is connected to a socket and stderr to a pipe.

    :param list args:
        Program argument vector.
    :param bool blocking:
        If :data:`True`, the sockets use blocking IO, otherwise non-blocking.
    :param int size:
        If not :data:`None`, use the value as the socket buffer size.
    :param function preexec_fn:
        If not :data:`None`, a function to run within the post-fork child
        before executing the target program.
    :returns:
        :class:`PopenProcess` instance.
    """

    parent_rw_fp, child_rw_fp = mitogen.parent.create_socketpair(size=size, blocking=blocking)
    stderr_r, stderr = mitogen.core.pipe(blocking=blocking)
    mitogen.core.set_cloexec(stderr_r.fileno())
    try:
        proc = testlib.subprocess.Popen(
            args=args,
            stdin=child_rw_fp,
            stdout=child_rw_fp,
            stderr=stderr,
            close_fds=True,
            preexec_fn=preexec_fn,
        )
    except Exception:
        parent_rw_fp.close()
        stderr_r.close()
        raise
    finally:
        child_rw_fp.close()
        stderr.close()

    return mitogen.parent.PopenProcess(
        proc=proc,
        stdin=parent_rw_fp,
        stdout=parent_rw_fp,
        stderr=stderr_r,
    )


class DummyConnectionBlocking(mitogen.parent.Connection):
    """Dummy blocking IO connection"""

    create_child = staticmethod(create_child_using_sockets)
    name_prefix = "dummy_blocking"

    #: Dictionary of extra kwargs passed to :attr:`create_child`.
    #: Use a size smaller than the conn.get_preamble() size so multiple
    #: read-calls are needed in the first stage.
    create_child_args = {"blocking": True, "size": 4096}


class DummyConnectionNonBlocking(mitogen.parent.Connection):
    """Dummy non-blocking IO connection"""

    create_child = staticmethod(create_child_using_sockets)
    name_prefix = "dummy_non_blocking"

    #: Dictionary of extra kwargs passed to :attr:`create_child`.
    #: Use a size smaller than the conn.get_preamble() size so multiple
    #: read-calls are needed in the first stage.
    create_child_args = {"blocking": False, "size": 4096}


class DummyConnectionEOFRead(mitogen.parent.Connection):
    """Dummy connection that triggers an EOF-read(STDIN) in the first_stage"""

    name_prefix = "dummy_eof_read"

    #: Dictionary of extra kwargs passed to :attr:`create_child`.
    create_child_args = {"blocking": True}

    @staticmethod
    def create_child(*a, **kw):
        proc = create_child_using_pipes(*a, **kw)
        # Close the pipe -> results in an EOF-read(STDIN) in the first_stage
        proc.stdin.close()
        # Whatever the parent writes to the child, drop it.
        proc.stdin = open("/dev/null", "wb")
        return proc


class DummyConnectionEndlessBlockingRead(mitogen.parent.Connection):
    """Dummy connection that triggers a non-returning read(STDIN) call in the
    first_stage.

    """

    name_prefix = "dummy_endless_blocking_read"

    #: Dictionary of extra kwargs passed to :attr:`create_child`.
    create_child_args = {"blocking": True}

    @staticmethod
    def create_child(*a, **kw):
        proc = create_child_using_pipes(*a, **kw)
        # Keep the pipe open by having a reference to it, otherwise it would be
        # automatically closed by the garbage collector.
        proc._mitogen_test_orig_stdin = proc.stdin
        # Whatever the parent writes to the child, drop it -> read from STDOUT
        # blocks forever in the fork child as no data could be read.
        proc.stdin = open("/dev/null", "wb")
        return proc


class ConnectionTest(testlib.RouterMixin, testlib.TestCase):
    def test_non_blocking_stdin(self):
        """Test that first stage works with non-blocking STDIN

        The boot command should read the preamble from STDIN, write all ECO
        markers to STDOUT, and then execute the preamble.

        This test writes the complete preamble to non-blocking STDIN.

        1. Fork child reads from non-blocking STDIN
        2. Fork child writes all data as expected by the protocol.
        3. A context call works as expected.

        """
        with testlib.LogCapturer() as _:
            ctx = self.router._connect(DummyConnectionNonBlocking, connect_timeout=0.5)
            self.assertEqual(3, ctx.call(operator.add, 1, 2))

    def test_blocking_stdin(self):
        """Test that first stage works with blocking STDIN

        The boot command should read the preamble from STDIN, write all ECO
        markers to STDOUT, and then execute the preamble.

        This test writes the complete preamble to blocking STDIN.

        1. Fork child reads from blocking STDIN
        2. Fork child writes all data as expected by the protocol.
        3. A context call works as expected.

        """
        with testlib.LogCapturer() as _:
            ctx = self.router._connect(DummyConnectionBlocking, connect_timeout=0.5)
            self.assertEqual(3, ctx.call(operator.add, 1, 2))

    def test_broker_connect_eof_error(self):
        """Test that broker takes care about EOF errors in the first stage

        The boot command should write an ECO marker to stdout, try to read the
        preamble from STDIN. This read returns with an EOF and the process exits.

        This test writes closes the pipe for STDIN of the fork child to enforce an EOF read call.
        1. Fork child reads from STDIN and reads an EOF and breaks the read-loop
        2. Decompressing the received data results in an error
        3. The child process exits
        4. The streams get disconnected -> mitogen.parent.EofError is raised

        """

        with testlib.LogCapturer() as _:
            e = self.assertRaises(mitogen.parent.EofError,
                              self.router._connect, DummyConnectionEOFRead, connect_timeout=0.5)
            self.assertIn("Error -5 while decompressing data", str(e))

            # Test that a TimeoutError is raised by the broker and all resources
            # are cleaned up.
            options = mitogen.parent.Options(
                old_router=self.router,
                max_message_size=self.router.max_message_size,
                connect_timeout=0.5,
            )
            conn = DummyConnectionEOFRead(options, router=self.router)
            e = self.assertRaises(mitogen.parent.EofError,
                                  conn.connect, context=mitogen.core.Context(None, 1234))
            self.assertIn("Error -5 while decompressing data", str(e))
            # Ensure the child process is reaped if the connection times out.
            testlib.wait_for_child(conn.proc.pid)
            e = self.assertRaises(OSError,
                                  os.kill, conn.proc.pid, 0)
            self.assertEqual(e.args[0], errno.ESRCH)

    def test_broker_connect_timeout_because_endless_blocking_read(self):
        """Test that broker takes care about connection timeouts

        The boot command should write an ECO marker to stdout, try to read the
        preamble from STDIN. This read blocks forever as the parent does write
        all the data to /dev/null instead of the pipe. The broker should then
        raise a TimeoutError as the child needs too much time.

        This test writes no data to STDIN of the fork child to enforce a blocking read call.
        1. Fork child tries to read from STDIN, but blocks forever.
        2. Parent connection timeout timer pops up and the parent cleans up
           everything from the child (e.g. kills the child process).
        3. TimeoutError is raised in the connect call

        """
        with testlib.LogCapturer() as _:
            # Ensure the child process is reaped if the connection times out.
            options = mitogen.parent.Options(
                old_router=self.router,
                max_message_size=self.router.max_message_size,
                connect_timeout=0.5,
            )

            conn = DummyConnectionEndlessBlockingRead(options, router=self.router)
            try:
                self.assertRaises(mitogen.core.TimeoutError,
                                  lambda: conn.connect(context=mitogen.core.Context(None, 1234))
                                  )
                testlib.wait_for_child(conn.proc.pid)
                e = self.assertRaises(OSError,
                                      os.kill, conn.proc.pid, 0)
                self.assertEqual(e.args[0], errno.ESRCH)
            finally:
                conn.proc._mitogen_test_orig_stdin.close()


class CommandLineTest(testlib.RouterMixin, testlib.TestCase):
    # Ensure this version of Python produces a command line that is sufficient
    # to bootstrap this version of Python.
    #
    # TODO:
    #   * 2.7 starting 2.4
    #   * 2.7 starting 3.x
    #   * 3.x starting 2.7

    def test_valid_syntax(self):
        """Test valid syntax

        The boot command should write an ECO marker to stdout, read the
        preamble from stdin, then execute it.

        This test attaches /dev/zero to stdin to create a specific failure

        1. Fork child reads <compressed preamble size> bytes of NUL (`b'\0'`)
        2. Fork child crashes (trying to decompress the junk data)
        3. Fork child's file descriptors (write pipes) are closed by the OS
        4. Fork parent does `dup(<read pipe>, <stdin>)` and `exec(<python>)`
        5. Python reads `b''` (i.e. EOF) from stdin (a closed pipe)
        6. Python runs `''` (a valid script) and exits with success

        """

        options = mitogen.parent.Options(max_message_size=123)
        conn = mitogen.parent.Connection(options, self.router)
        conn.context = mitogen.core.Context(None, 123)
        fp = open("/dev/zero", "rb")
        try:
            proc = testlib.subprocess.Popen(
                args=conn.get_boot_command(),
                stdin=fp,
                stdout=testlib.subprocess.PIPE,
                stderr=testlib.subprocess.PIPE,
            )
            stdout, stderr = proc.communicate()
            self.assertEqual(0, proc.returncode)
            self.assertEqual(stdout,
                mitogen.parent.BootstrapProtocol.EC0_MARKER+b('\n'))
            self.assertIn(
                b("Error -3 while decompressing data"),  # Unknown compression method
                stderr,
            )
        finally:
            fp.close()

    def test_premature_eof(self):
        """The boot command should write an ECO marker to stdout, read the
        preamble from stdin, then execute it.

        This test writes some data to STDIN and closes it then to create an
        EOF situation.
        1. Fork child tries to read from STDIN, but stops as EOF is received.
        2. Fork child crashes (trying to decompress the junk data)
        3. Fork child's file descriptors (write pipes) are closed by the OS
        4. Fork parent does `dup(<read pipe>, <stdin>)` and `exec(<python>)`
        5. Python reads `b''` (i.e. EOF) from stdin (a closed pipe)
        6. Python runs `''` (a valid script) and exits with success"""

        options = mitogen.parent.Options(max_message_size=123)
        conn = mitogen.parent.Connection(options, self.router)
        conn.context = mitogen.core.Context(None, 123)
        proc = testlib.subprocess.Popen(
            args=conn.get_boot_command(),
            stdout=testlib.subprocess.PIPE,
            stderr=testlib.subprocess.PIPE,
            stdin=testlib.subprocess.PIPE,
        )

        # Do not send all of the data from the preamble
        proc.stdin.write(conn.get_preamble()[:-128])
        proc.stdin.close()
        try:
            returncode = proc.wait(timeout=1)
        except testlib.subprocess.TimeoutExpired:
            proc.kill()
            proc.wait(timeout=3)
            self.fail("First stage did not handle EOF on STDIN")
        try:
            self.assertEqual(0, returncode)
            self.assertEqual(
                proc.stdout.read(),
                mitogen.parent.BootstrapProtocol.EC0_MARKER + b("\n"),
            )
            self.assertIn(
                b("Error -5 while decompressing data"),
                proc.stderr.read(),
            )
        finally:
            proc.stdout.close()
            proc.stderr.close()



import os

import mitogen.incus
import mitogen.parent

import testlib


class ConstructorTest(testlib.RouterMixin, testlib.TestCase):
    def test_okay(self):
        incus_path = testlib.data_path('stubs/stub-incus.py')
        context = self.router.incus(
            container='container_name',
            incus_path=incus_path,
        )

        argv = eval(context.call(os.getenv, 'ORIGINAL_ARGV'))
        self.assertEqual(argv[0], incus_path)
        self.assertEqual(argv[1], 'exec')
        self.assertEqual(argv[2], '--mode=non-interactive')
        self.assertEqual(argv[3], 'container_name')

    def test_eof(self):
        e = self.assertRaises(mitogen.parent.EofError,
            lambda: self.router.incus(
                container='container_name',
                incus_path='true',
            )
        )
        self.assertTrue(str(e).endswith(mitogen.incus.Connection.eof_error_hint))



# SPDX-FileCopyrightText: 2026 Mitogen authors <https://github.com/mitogen-hq>
# SPDX-License-Identifier: BSD-3-Clause
from __future__ import absolute_import, division, print_function
__metaclass__ = type

import itertools
import time

import ansible.plugins.callback
import ansible.utils.display

display = ansible.utils.display.Display()


class CallbackModule(ansible.plugins.callback.CallbackBase):
    CALLBACK_NAME = 'profile_plays'
    CALLBACK_NEEDS_WHITELIST = True
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_VERSION = 2.0

    def __init__(self):
        self._data = []
        self._play_counter = itertools.count()
        super(CallbackModule, self).__init__()

    def v2_playbook_on_play_start(self, play):
        start = time.time()
        if self._data:
            prev = self._data[-1]
            prev.update({
                'end': start,
                'duration': start - prev['start'],
            })

        serial = next(self._play_counter)
        self._data.append({
            'serial': serial,
            'name': play.get_name().strip() or u'PLAY %d' % serial,
            'uuid': play._uuid,
            'start': start,
        })

    def v2_playbook_on_stats(self, stats):
        start = time.time()
        if self._data:
            prev = self._data[-1]
            prev.update({
                'end': start,
                'duration': start - prev['start'],
            })

        data = self._data[:]
        data.sort(key=lambda d: d['duration'], reverse=True)

        display.display(u'{:=<79}'.format(u'Play durations '))
        for d in data[:10]:
            display.display(u'{name:70.70}  {duration:>6,.2f}s'.format(**d))



import socket

import mitogen.core

import testlib

class SocketPairTest(testlib.TestCase):
    def test_socketpair_blocking_unspecified(self):
        "Test that unspecified blocking arg (None) batches socket.socketpair()"
        sk_fp1, sk_fp2 = socket.socketpair()
        mi_fp1, mi_fp2 = mitogen.core.socketpair()

        self.assertEqual(mitogen.core.get_blocking(sk_fp1.fileno()),
                         mitogen.core.get_blocking(mi_fp1.fileno()))
        self.assertEqual(mitogen.core.get_blocking(sk_fp2.fileno()),
                         mitogen.core.get_blocking(mi_fp2.fileno()))
        mi_fp1.close()
        mi_fp2.close()
        sk_fp1.close()
        sk_fp2.close()

    def test_socketpair_blocking_true(self):
        mi_fp1, mi_fp2 = mitogen.core.socketpair(blocking=True)
        self.assertTrue(mitogen.core.get_blocking(mi_fp1.fileno()))
        self.assertTrue(mitogen.core.get_blocking(mi_fp2.fileno()))
        mi_fp1.close()
        mi_fp2.close()

    def test_socketpair_blocking_false(self):
        mi_fp1, mi_fp2 = mitogen.core.socketpair(blocking=False)
        self.assertFalse(mitogen.core.get_blocking(mi_fp1.fileno()))
        self.assertFalse(mitogen.core.get_blocking(mi_fp2.fileno()))
        mi_fp1.close()
        mi_fp2.close()




import logging
import sys
import unittest

try:
    from unittest import mock
except ImportError:
    import mock

import testlib
import mitogen.core
from mitogen.core import b

PY2 = sys.version_info[0] == 2
if PY2:

    def logging_getLogRecordFactory():
        return logging.LogRecord

    def logging_setLogRecordFactory(factory):
        logging.LogRecord = factory

else:
    logging_getLogRecordFactory = logging.getLogRecordFactory
    logging_setLogRecordFactory = logging.setLogRecordFactory


def ping():
    pass


def log_test():
    logging.getLogger(__name__).info("This is a test")


class BufferingTest(testlib.TestCase):
    klass = mitogen.core.LogHandler

    def record(self):
        return logging.LogRecord(
            name='name',
            level=99,
            pathname='pathname',
            lineno=123,
            msg='msg',
            args=(),
            exc_info=None,
        )

    def build(self):
        context = mock.Mock()
        return context, self.klass(context)

    def test_initially_buffered(self):
        context, handler = self.build()
        rec = self.record()
        handler.emit(rec)
        self.assertEqual(0, context.send.call_count)
        self.assertEqual(1, len(handler._buffer))

    def test_uncork(self):
        context, handler = self.build()
        rec = self.record()
        handler.emit(rec)
        handler.uncork()

        self.assertEqual(1, context.send.call_count)
        self.assertEqual(None, handler._buffer)

        _, args, _ = context.send.mock_calls[0]
        msg, = args

        self.assertEqual(mitogen.core.FORWARD_LOG, msg.handle)
        self.assertEqual(b('name\x0099\x00msg'), msg.data)


class StartupTest(testlib.RouterMixin, testlib.TestCase):
    def test_earliest_messages_logged(self):
        log = testlib.LogCapturer()
        log.start()

        c1 = self.router.local()
        c1.shutdown(wait=True)

        logs = log.stop()
        self.assertIn('Python version is', logs)
        self.assertIn('Parent is context 0 (master)', logs)

    def test_earliest_messages_logged_via(self):
        c1 = self.router.local(name='c1')
        # ensure any c1-related msgs are processed before beginning capture
        c1.call(ping)

        log = testlib.LogCapturer()
        log.start()

        c2 = self.router.local(via=c1, name='c2', debug=True)
        c2.shutdown(wait=True)

        logs = log.stop()
        self.assertIn('Python version is', logs)

        expect = 'Parent is context %s (%s)' % (c1.context_id, 'parent')
        self.assertIn(expect, logs)


class LogRecordFactoryTest(testlib.RouterMixin, testlib.TestCase):
    def setUp(self):
        super(LogRecordFactoryTest, self).setUp()
        self.original_factory = logging_getLogRecordFactory()

    def tearDown(self):
        logging_setLogRecordFactory(self.original_factory)
        super(LogRecordFactoryTest, self).tearDown()

    def test_logrecordfactory(self):
        # Change logging factory and add a custom attribute
        old_factory = logging_getLogRecordFactory()

        def record_factory(*args, **kwargs):
            record = old_factory(*args, **kwargs)
            record.custom_attribute = 0xDEADBEEF
            return record

        logging_setLogRecordFactory(record_factory)
        log = testlib.LogCapturer(
            formatter=logging.Formatter("%(custom_attribute)x - %(message)s"),
        )
        log.start()
        c1 = self.router.local(name="c1")
        c1.call(log_test)
        logs = log.stop()
        self.assertIn("deadbeef - This is a test", logs)


StartupTest = unittest.skipIf(
    condition=sys.version_info < (2, 7) or sys.version_info >= (3, 6),
    reason="Message log flaky on Python < 2.7 or >= 3.6"
)(StartupTest)
